<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Util\Notification\Facades\Notification as EventPortalNotification;
use EventPortal\Domain\Util\Notification\SystemNotification;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Notification as LaravelNotification;
use Spatie\Snapshots\MatchesSnapshots;

class NotificationDataTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function a_notification_can_be_sent_to_a_recipient()
    {
        LaravelNotification::fake();

        $expectedRecipient = factory(User::class)
            ->state('snapshot')
            ->create();

        EventPortalNotification::make('test.notification')
            ->to($expectedRecipient)
            ->send();

        LaravelNotification::assertSentTo(
            $expectedRecipient,
            SystemNotification::class
        );
    }
    /** @test **/
    public function the_event_can_be_retrieved()
    {
        $expectedEvent = factory(Event::class)
            ->create();

        $notification = EventPortalNotification::make('event.invite')
            ->event($expectedEvent);

        $this->assertSameModel($expectedEvent, $notification->data->getEvent());
    }
}
